//+------------------------------------------------------------------+
//|                                    Spearman Rank Correlation.mq4 |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers  1
#property indicator_color1   PaleVioletRed
#property indicator_width1   2
#property indicator_minimum -1
#property indicator_maximum  1

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    Rank            = 14;
extern int    Price           = PRICE_CLOSE;
extern double LevelUp         =  0.75;
extern double LevelDn         = -0.75;
extern int    BarsToCount     = 1000;
extern bool   Interpolate     = true;
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

double buffer1[];
double buffer2[];
double trend[];
double PR[][3];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(3);          
   SetIndexBuffer(0, buffer1);
   SetIndexBuffer(1, buffer2);
   SetIndexBuffer(2, trend);
   
      //
      //
      //
      //
      //
      
         Rank = MathMin(MathMax(Rank,10),100);
         ArrayResize(PR,Rank);
            if (BarsToCount > 0)
                BarsToCount = MathMax(BarsToCount,250);
         indicatorFileName = WindowExpertName();
         calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);

         SetLevelValue(0,LevelUp);          
         SetLevelValue(1,LevelDn);          
         
      //
      //
      //
      //
      //
          
   IndicatorShortName(timeFrameToString(timeFrame)+" Spearman (" + Rank + ")");
   return(0);
}
int deinit()
{
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   double coef         = MathPow(Rank,3) - Rank;
   int    counted_bars = IndicatorCounted();
   int    i,k,n,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { buffer1[0] = MathMin(limit+1,Bars-1); return(0); }
         if (BarsToCount > 0) limit = MathMin(limit,BarsToCount);

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame == Period())
   {
      for(i = limit; i >= 0; i--)
      {
         buffer2[i] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
            for(k=0; k<Rank; k++)
            {
                  PR[k,0] = buffer2[i+k];
                  PR[k,1] = k+1;
            }
            SortIt();         

            //
            //
            //
            //
            //
                     
            double sum = 0.0;
            for(k=0; k<Rank; k++)
               sum += MathPow(PR[k,1]-PR[k,2],2);
               trend[i]   = trend[i+1];
               buffer1[i] = 1.00-6.00*sum/coef;
                  if (buffer1[i]>LevelUp)                       trend[i]= 1;
                  if (buffer1[i]<LevelDn)                       trend[i]=-1;
                  if (buffer1[i]<LevelUp && buffer1[i]>LevelDn) trend[i]= 0;

      }      
      manageAlerts();
      return(0);
   }      
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (BarsToCount > 0) limit = MathMin(limit,BarsToCount);
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         buffer1[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Rank,Price,LevelUp,LevelDn,BarsToCount,0,y);
         trend[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Rank,Price,LevelUp,LevelDn,BarsToCount,2,y);

         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(k = 1; k < n; k++)
               buffer1[i+k] = buffer1[i] + (buffer1[i+n]-buffer1[i])*k/n;
   }
   manageAlerts();
   return(0);
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar]   ==  1) doAlert(whichBar,DoubleToStr(LevelUp,2)+" broken up");
         if (trend[whichBar]   == -1) doAlert(whichBar,DoubleToStr(LevelDn,2)+" broken down");
         if (trend[whichBar+1] == -1) doAlert(whichBar,DoubleToStr(LevelDn,2)+" broken up");
         if (trend[whichBar+1] ==  1) doAlert(whichBar,DoubleToStr(LevelUp,2)+" broken down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS),"spearman level ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Spearman rank"),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//    shell sort 
//
//
//

void SortIt()
{
   int    i,j,increment = Rank / 2;
   double temp,temi;

   //
   //
   //
   //
   //
     
   while (increment > 0)
   {
      for (i = increment; i < Rank; i++)
      {
         j    = i;
         temp = PR[i,0];
         temi = PR[i,1];
         while ((j >= increment) && (PR[j-increment,0] < temp))
         {
            PR[j,0] = PR[j-increment,0];
            PR[j,1] = PR[j-increment,1];
            j       =    j-increment;
         }
         PR[j,0] = temp;
         PR[j,1] = temi;
      }
      if (increment == 2) increment = 1;
      else                increment = (increment / 2.2);
   }

   //
   //
   //
   //
   //
   //
   //

   for(i = 0; i < Rank; i++)
   {
      PR[i,2] = i+1; if(PR[i,0] != PR[i+1,0]) continue;

      //
      //
      //    average where needed
      //
      //
      
            temp = i+1;
                for (j=i+1, temi=1; j < Rank; j++,temi++)
                   if(PR[j,0] == PR[i,0])
                         temp += (j+1);
                   else break;
            temp /= temi;
      for(; i < j; i++) PR[i,2] = temp;
   }
}